use crate::helpers::jwt::get_encoding_key;
use crate::middlewares::error::ErrorResponse;
use crate::models::user::Users;
use actix_web::http::StatusCode;
use actix_web::{post, web, HttpResponse};
use chrono::{Days, Utc};
use jsonwebtoken::{encode, Header};
use serde::{Deserialize, Serialize};

#[derive(Debug, Deserialize, Serialize)]
pub struct JWTClaims {
    pub user_id: String,
    pub exp: usize,
}

#[post("/auth/login")]
async fn login(body: web::Json<LoginBody>) -> Result<HttpResponse, ErrorResponse> {
    #[derive(Deserialize, Serialize)]
    struct Response {
        access_token: String,
        exp: usize,
    }

    let user = Users::find_by_email(&body.email);
    match user {
        Ok(user) => match Users::verify_password(&body.password, &user.password) {
            Ok(_res) => {
                let exp = Utc::now()
                    .checked_add_days(Days::new(30))
                    .expect("valid timestamp")
                    .timestamp();

                let claims = JWTClaims {
                    user_id: user.id.to_string(),
                    exp: exp as usize,
                };
                let token = encode(&Header::default(), &claims, &get_encoding_key()).unwrap();

                Ok(HttpResponse::Ok().json(Response {
                    access_token: token,
                    exp: exp as usize,
                }))
            }
            Err(_e) => {
                return Err(ErrorResponse {
                    message: "Invalid credentials.".to_string(),
                    status: StatusCode::BAD_REQUEST,
                })
            }
        },
        Err(_err) => {
            return Err(ErrorResponse {
                message: "Invalid credentials.".to_string(),
                status: StatusCode::BAD_REQUEST,
            })
        }
    }
}

#[derive(Deserialize)]
struct LoginBody {
    email: String,
    password: String,
}
