use crate::services::service::Service;
use actix_web::http::Method;
use async_trait::async_trait;
use reqwest::{Client, RequestBuilder};
use serde::Deserialize;

#[derive(Deserialize)]
struct SpotifyResponse {
    access_token: String,
}

#[derive(Clone)]
pub struct Spotify {
    token: String,
}

pub async fn instance<'a>() -> Spotify {
    Spotify::new().await
}

impl Spotify {
    pub async fn new() -> Spotify {
        let mut spotify = Spotify {
            token: String::new(),
        };
        spotify.fetch_token().await;
        spotify
    }

    fn api(&self, url: &str, method: Method) -> RequestBuilder {
        let client = Client::new();

        client
            .request(method, &format!("https://api.spotify.com/v1{url}"))
            .bearer_auth(self.get_token())
    }

    // pub async fn get_track(self, spotify_id: &str) {
    //     let spotify = &SPOTIFY;
    // }

    pub async fn get_artist(self, spotify_id: &str) {}

    pub async fn get_track_by_isrc(self, isrc: &str) {}

    pub async fn search(self, search: &str) {}
}

#[async_trait]
impl Service for Spotify {
    fn get_token(&self) -> &String {
        &self.token
    }

    async fn fetch_token(&mut self) -> &String {
        let response = Client::new()
            .post("https://accounts.spotify.com/api/token")
            .basic_auth(
                get_env("SPOTIFY_CLIENT_ID"),
                Some(get_env("SPOTIFY_CLIENT_SECRET")),
            )
            .form(&[("grant_type", "client_credentials")])
            .send()
            .await;

        match response {
            Ok(res) => {
                let data: Result<SpotifyResponse, _> = res.json().await;

                self.token = data.unwrap().access_token.into();
                &self.token
            }
            Err(_) => {
                panic!("Invalid response (Spotify Token)")
            }
        }
    }
}

fn get_env(key: &str) -> String {
    std::env::var(key).unwrap()
}
